var app = Vue.createApp({
    data() {
        return {
            // Shared variables between the children.
            showManualSimulationLimitPicker: false,
            types_input_row: [],
            scanner_input_row: [],
            types_output_rows: [],
            start_time: '',
            end_time: '',
            simulation_output_rows: []
        }
    },
    mounted() {
        // Initialize the datetimepickers

        // They require their own callback as v-model does not naitively support
        // its regular update method.
        var external_this = this
        console.log("Mounted")
        $('#startTime').datetimepicker({
            onClose:function() {
                external_this.start_time = this.datetimepicker('getValue').getValue().toISOString()
            }
        });
        $('#endTime').datetimepicker({
            onClose:function() {
                external_this.end_time = this.datetimepicker('getValue').getValue().toISOString()
            }
        });
    },
    methods: {
        // Parse a simulation response from the API to rows that can be
        // read by the simulate-results component
        // @arg r The result from the API.
        parse_simulation_result(r) {
            var new_output = []
            Object.entries(r).forEach(entry => {
                const [k,v] = entry;

                v.forEach(function(row, i) {
                    new_output.push({
                        argosname: k,
                        argosname_rowspan: v.length,
                        new_class: row[0],
                        old_limit: row[1],
                        new_limit: row[2],
                        increase: row[3],
                        percent: row[4],
                        scanner: row[5],
                        show_img: false
                    })
                })
            })

            this.simulation_output_rows = new_output
        },
        //onclick from the suggest simulate button to request the
        //data from the API.
        suggest_simulate_subtypes() {
            this.showManualSimulationLimitPicker=false
            var external_this = this

            var payload = {
                start_time: this.start_time,
                end_time: this.end_time,
                subtypes: this.types_input_row,
                scanners: this.scanner_input_row
            }

            $.post("/simulate_images", JSON.stringify(payload), function (r) {
                external_this.parse_simulation_result(r)
            });
        },
        //onclick from the manual simulate button to request the data from the
        //API.
        manual_simulate(data) {
            var external_this = this

            var payload = {
                start_time: this.start_time,
                end_time: this.end_time,
                limits: data,
                scanners: this.scanner_input_row
            }
            console.log("Simulating...")
            $.post("/simulate_images/manual", JSON.stringify(payload), function(r) {
                console.log(r)
                external_this.parse_simulation_result(r)
            })
        },
        // onclick for the button that requests argosnames and old limits from
        // the API.
        manual_simulate_argosnames() {
            var payload = {subtypes: this.types_input_row}
            var result = []
            var external_this = this

            $.post("/manual_simulate_rows",data=JSON.stringify(payload), function(r) {
                // For each Subtype...
                Object.entries(r).forEach(function (entry) {
                    const [subtype, v] = entry;

                    // For each ArgosName...
                    Object.entries(v).forEach(function (argos_name_dict, subtype_i) {
                        const [argos_name, limit_dict] = argos_name_dict;

                        // For each classlimit...
                        Object.entries(limit_dict).forEach(function (limit, limit_i) {
                            const [limit_class, value] = limit;

                            result.push({
                                subtype:subtype,
                                argosname:argos_name,
                                class_limit:limit_class,
                                old_value:value,
                                new_value:value
                            })
                        })
                    })
                })

                external_this.types_output_rows = result;
                external_this.showManualSimulationLimitPicker=true;
            })

        }
    }
})

app.component("dropdown-menu", {
    data() {
        return {
            rows: [],
            inputText:'',
            dropdownData:[],
            inputId:this.input_id
        }
    },
    // Initialize the dropdown menu with data from the API.
    created() {
        var external_this = this;
        $.get(this.dropdown_get_endpoint, function (r) {
            this.dropdownData = r;
            var dropdown_element = $(`#${external_this.selectFieldId}`);
            dropdown_element.select2({data:r, width:"300px"}).trigger("change")
            dropdown_element.on('select2:close', function (event) { this.value = null })
            dropdown_element.on("select2:select", function (e) {
                // Some elements are designed to add their entire group.
                // They are structured by delimiting each sub-id with <**>.
                // Split these and add the sub-ids individually.
                group_index = e.params.data.id.search("<\\*\\*>");
                if (group_index > 0) {
                    var group = e.params.data.id
                    console.log("Adding all in group " + group);
                    var argosnames = group.split("<**>");
                    $.each(argosnames, function (k, v) {
                        external_this.add(v);
                    });
                } else {
                    external_this.add(e.params.data.id);
                }
            });
        });
    },
    emits: ['row'],
    methods: {
        remove(i) {
            this.rows.splice(i, 1)
            // Export rows so the parent knows that it has changed.
            this.$emit('row', this.rows)
        },
        add(e) {
            if (this.rows.indexOf(e) === -1) {
                this.rows.push(e)
            }
            // Export rows so the parent knows that it has changed.
            this.$emit('row', this.rows)
        },
    },
    computed: {
        selectFieldId() {
            return this.inputid + 'SelectField'
        }
    },
    props: ['inputid', 'dropdown_get_endpoint', 'dropdown_label'],
    template: `
        <label :for="selectFieldId">{{dropdown_label}}</label>
        <select :id="selectFieldId">
        <option></option>
        </select>

        <table>
            <template v-for="(row, i) in rows">
                <tr>
                    <td>
                        <button v-on:click="remove(i)">Remove</button>
                    </td>
                    <td>{{row}}</td>
                </tr>
            </template>
        </table>
        `
})

app.component("manual-simulate-picker", {
    data() {
        return {
        }
    },

    methods: {
        // Get the required data for manual simulation from the table.
        format_manual_simulate() {
            var limits = {}

            // Restructure each row into arrays of argosnames and classlimits.
            Object.entries(this.rows).forEach(function(entry) {
                const [k,row] = entry
                if (limits[row.argosname] === undefined) {
                    limits[row.argosname] = {}
                }
                limits[row.argosname][row.class_limit] = row.new_value
            })

            this.$emit('simulate', limits)
        },
        // Updating an element inside a dict in a list is not supported
        // naitively.
        updateNewValue(el, i) {
            this.rows[i].new_value = parseInt(el.target.value)
        }
    },
    props: ['rows'],

    template: `
    <table id="manualSimulateTable" class="table table-striped">
        <tbody>
            <tr>
                <th>Subtype</th>
                <th>Argos Name</th>
                <th>Class limit</th>
                <th>Old value</th>
                <th>New value</th>
            </tr>
            
            <template v-for="(row, i) in rows">
                <tr>
                    <td>{{row.subtype}}</td> 
                    <td>{{row.argosname}}</td> 
                    <td>{{row.class_limit}}</td> 
                    <td>{{row.old_value}}</td> 
                    <td>
                        <input type="number" @change="updateNewValue($event, i)"> 
                    </td> 
                </tr>
            </template>
        </tbody>
    </table>
    
    <button v-on:click="format_manual_simulate()">Simuler</button>
    `
})

app.component("simulate-results", {
    data() {
        return {
        }
    },
    props: ['rows'],
    methods: {
        get_pareto_url(row) {
            return `${row.argosname}${row.new_class}.png`
        },
        show_pareto_chart(row) {
            row.show_img = true
        }
    },
    template: `
    <table id='simulateResults' class="table table-striped">
        <tbody>
            <tr>
              <th>Argos Name</th>
              <th>New class</th>
              <th>Old limit</th>
              <th>New limit</th>
              <th>Production increase</th>
              <th>%</th>
              <th>Scanner</th>
              <th></th>
            </tr>
            <template v-for="(row, i) in rows">
                <tr>
                    <td v-if="i === 0" :rowspan="row.argosname_rowspan">{{row.argosname}}</td>
                    <td>{{row.new_class}}</td>
                    <td>{{row.old_limit}}</td>
                    <td>{{row.new_limit}}</td>
                    <td>{{row.increase}}</td>
                    <td>{{row.percent}}</td>
                    <td>{{row.scanner}}</td>
                    <td>
                        <button v-on:click="show_pareto_chart(row)">Pareto chart</button>
                        <img :src='get_pareto_url(row)' v-if='row.show_img' alt="">
                    </td>
                </tr>
            </template>
        </tbody>
    </table>
    `
})

const dropdown_lists = app.mount("#parameterSelection")
