"""External plugins for the plugin architecture.

Specified plugins must be decorated with\
:meth:`ArgosAnalysePlatform.plugin_manager.PluginManager.register_plugin`
Import an instance of this with\
``from ArgosAnalysePlatform.plugin_manager import plugmanager``

"""
import logging
from importlib import import_module
from pathlib import Path

logger = logging.getLogger(__name__)


def import_plugins(target_directory: Path, relative_to: Path = None):
    """import each Python file that is in the specified directory.

    :param target_directory: The target directory to load plugins from
    :param relative_to: In some cases, it might be needed to explicitly specify\
        which path the files are relative to. If needed, specify here.

    """

    logger.debug(f"CWD: {Path.cwd()}")
    logger.debug(f"relative_to: {relative_to}")

    root_package_name = __name__.split(".", 1)[0]
    logger.debug(f"Root package name: {root_package_name}")
    for f in target_directory.absolute().iterdir():
        if f.suffix == ".py":
            relative = f.relative_to(relative_to or Path.cwd())
            import_str = ".".join(relative.parts)[:-3]
            logger.debug(f"Importing {import_str}")
            import_module(import_str)
