"""Utility functions for the entire project. """
from typing import List, Any, Callable, Tuple
import itertools


def groupby(data: List[Any], key: Callable) -> List[Tuple[Any, List[Any]]]:
    """Group data by key.

    Group data by key, then format the result as a list of (key, list(values))

    :param data: The data to group
    :param key: A callable to determine which field in each element to
    group on.

    :return: A list of tuples with the key and a list of matched entries.

    """
    return [
        (k, list(v))  # k = class and v = (count, time).
        for k, v in itertools.groupby(sorted(data, key=key), key=key)
    ]
