# Installation

## Pip
```
pip install poetry
poetry install
```

# Usage

The API endpoints are reachable on localhost:8000. 


# API

The complete API endpoint documentation can be found on /docs. 

The API is currently able to serve data via a Grafana friendly and
Microsoft report builder friendly datastructure

## Grafana

POST `/search`: Takes no parameters, returns a list of avaliable metrics. 

POST `/query`: Takes a request for data with the following required parameters:

* `start_time` in iso format
* `end_time` in iso format
* `intervalMs` interval of timeseries buckets in milliseconds.
* `targets`: A list of dicts with the keys `target`. Must be the METRICS that are requested.
each target must be defined as a plugin with the same name.
* `scopedVars['scannere']['values']`: A dict(dict(list)) of scanners.

Plugins with the default category (`''`) is avaliable through `/query`.
Plugins registered to this endpoint must return a list of objects that will
be extended to the final result.

## MS Report builder

The report builder works slightly differently, in that it takes `www-form` as
input instead of json. It also expects XML as return format instead of JSON.

Therefore, the following endpoints are used:

POST `/search` with the parameter `category="report_builder"`. This will get
all avaliable plugins

POST `/xml/query` with the following parameters

* `action`: The plugin to use. Call several by comma separating. Do NOT 
  separate with a space.
  
* `start_time`: Time in isoformat (YYYY-MM-dd HH:mm:ss)
* `end_time`: Time in isoformat (YYYY-MM-dd HH:mm:ss)
* `resolution`: Resolution in minutes.
* `targets`: The scanners to target for this query.

Plugins with the category `report_builder` is accessible through `/xml/query`.
Plugins registered to this endpoint must return a single `ET.Element` object,
which will be appended to a root node.

# Plugins

Each `target` that is requested must be registered as a name of a plugin. This is done via
the `@plugmanager.register_plugin(name)` decorator. 

The decorated function must take the following arguments:

* `session`: The session to query the database with
* `name`: The name of the scanner
* `start_time`, `end_time`: The respective start and end times. 
* `resolution`: The resolution in minutes.

And the following keyword argument
* `extra_data`: Any extra data that were added.

```py
from ArgosAnalysePlatform import plugmanager

@plugmanager.register_plugin("test")
def func(session, name, start_time, end_time, resolution,extra_data):
  ...
```

## Categories

Plugins may be registered with a category to logically segregate functionaly,
and to indicate to the user which plugins are related. Specify the category
in `register_plugin` with the `category` parameter.

```py
from ArgosAnalysePlatform import plugmanager

@plugmanager.register_plugin("test", category='report_builder')
def func(session, name, start_time, end_time, resolution,extra_data):
  ...
```