# These pylint warnings are ignored as they are not that useful in
# tests

# pylint: disable=missing-function-docstring
# pylint: disable=missing-class-docstring
# pylint: skip-file
"""Testing the API methods"""

import xml.etree.ElementTree as ET
from copy import copy

import pytest

import ArgosAnalysePlatform


def test_base(client):
    response = client.get("/")

    assert response.json() == {}
    assert response.status_code == 200


def test_search(client, monkeypatch):
    def mock_get_plugins(*_):
        return ["a"]

    monkeypatch.setattr(
        ArgosAnalysePlatform.plugmanager, "get_plugins", mock_get_plugins
    )
    response = client.post("/search", json={})

    assert response.json() == ["a"]
    assert response.status_code == 200


def test_get_categories(monkeypatch, client):
    mocked_plugins = {"test": {"test1": 0, "test2": 0}}
    monkeypatch.setattr(ArgosAnalysePlatform.plugmanager, "_plugins", mocked_plugins)

    response = client.get("/plugins/categories")

    assert response.json() == {"test": ["test1", "test2"]}
    assert response.status_code == 200


def test_query(monkeypatch, client):
    @ArgosAnalysePlatform.plugmanager.register_plugin("test")
    def test(session, name, start, end, resolution, extra_data):
        # pylint: disable=unused-variable
        # pylint: disable=too-many-arguments
        return [{"target": name, "datapoints": [(1, 2)]}]

    from_date = "2020-02-20T00:00:00"
    to_date = "2020-02-20T01:00:00"

    response = client.post(
        "/query",
        json={
            "range": {"from": from_date, "to": to_date},
            "intervalMs": 1000 * 60,
            "targets": [{"target": "test", "data": "some_data"}],
            "scopedVars": {"scannere": {"value": ["scanner_name"]}},
        },
    )

    assert response.json() == [
        {"target": "scanner_name", "datapoints": [[1, "1970-01-01T00:00:02+00:00"]]}
    ]

    assert response.status_code == 200


xml_form_data = {
    "actions": "plater",
    "targets": "scanner_name",
    "start_time": "2021-01-01 00:00:00",
    "end_time": "2021-01-01 01:00:00",
    "resolution": "0",
}


def test_xml_query(client):
    @ArgosAnalysePlatform.plugmanager.register_plugin(
        "plater", category="report_builder"
    )
    def plugin(_, name, start, end, res, extra_data):
        # pylint: disable=unused-variable
        node = ET.Element("Node")
        ET.SubElement(node, "actions").text = "plater"
        ET.SubElement(node, "targets").text = str(name)
        ET.SubElement(node, "start_time").text = str(start)
        ET.SubElement(node, "end_time").text = str(end)
        ET.SubElement(node, "resolution").text = str(res)
        ET.SubElement(node, "extra_data").text = str(extra_data)

        return node

    response = client.post(
        "/xml/query",
        data=xml_form_data,
    )

    response_tree = ET.fromstring(response.text)

    for k, v in xml_form_data.items():
        assert response_tree.find(f"Node/{k}").text == v

    assert response_tree.find("Node/extra_data").text == str(xml_form_data)
    assert response.status_code == 200


def test_xml_query_not_returns_xmltree(client):
    @ArgosAnalysePlatform.plugmanager.register_plugin(
        "test_raises", category="report_builder"
    )
    def test(*args, **kwargs):
        return "hei"

    form_data = copy(xml_form_data)
    form_data["actions"] = "test_raises"

    with pytest.raises(ValueError):
        response = client.post("/xml/query", data=form_data)
