import pytest
from ArgosAnalysePlatform.database import DatabaseConnections
from sqlalchemy.orm import Session

scanner_data = {
    "username": "a",
    "password": "b",
    "ip": "1.1.1.1",
    "tablename": "c",
}

scanner_data_2 = {
    "username": "a2",
    "password": "b2",
    "ip": "1.1.1.2",
    "tablename": "c2",
}

scanner_exp_data = {
    "username": "a",
    "password": "b",
    "host": "1.1.1.1",
    "database": "c",
}

scanner_exp_data_2 = {
    "username": "a2",
    "password": "b2",
    "host": "1.1.1.2",
    "database": "c2",
}


@pytest.mark.parametrize(
    "inp, expected_out",
    (
        (
            {
                "connections": {
                    "scanner1": scanner_data,
                    "scanner2": scanner_data_2,
                },
            },
            {
                "scanner1": scanner_exp_data,
                "scanner2": scanner_exp_data_2,
            },
        ),
        (
            {},
            {},
        ),
    ),
)
def test_init(inp, expected_out):
    result = DatabaseConnections(inp)

    assert len(expected_out) == len(result.connections)
    for sc_name, data in expected_out.items():
        engine_url = result.connections[sc_name].get("engine")
        for k, v in data.items():
            assert getattr(engine_url.url, k) == v


def test_sessioniter():
    inp_dct = {"connections": {"scanner1": scanner_data, "scanner2": scanner_data_2}}
    database = DatabaseConnections(inp_dct)

    for name, session in database.sessioniter(["scanner1", "scanner2"]):
        assert name in inp_dct["connections"]
        assert isinstance(session, Session)
        assert session.bind.url.username == inp_dct["connections"][name]["username"]
