# These pylint warnings are ignored as they are not that useful in
# tests
# pylint: skip-file
"""Test plugin manager"""

from fastapi import HTTPException
import pytest

from ArgosAnalysePlatform.plugin_manager import PluginManager


def test_register_plugin():
    manager = PluginManager()

    @manager.register_plugin("plug_name", category="test_category")
    def test(*args, **kwargs):
        pass

    assert "plug_name" in manager._plugins["test_category"]


def test_call_plugin():
    manager = PluginManager()
    called = False

    @manager.register_plugin("plug_name")
    def test(arg1):
        nonlocal called
        called = True
        return arg1

    result = manager.call_plugin("plug_name", "")("argument")
    assert called
    assert result == "argument"


def test_get_plugin_not_exists():
    manager = PluginManager()

    try:
        manager.call_plugin("Not exists", category="")

        assert False, "Call plugin should have raised 404"
    except HTTPException as err:
        if err.status_code != 404:
            assert False, "Call plugin should have raised 404."


def test_get_plugin_catches_exceptions():
    manager = PluginManager()

    @manager.register_plugin("test_raises_unknown", "")
    def test():
        raise ValueError("Unknown error")

    try:
        manager.call_plugin("test_raises_unknown", "")()
        assert False, "Should have raised 500"
    except ValueError:
        assert False, "Should have raised HTTPException"
    except HTTPException as err:
        if err.status_code != 500:
            assert False, "Should have raised 500."


def test_get_plugins():
    manager = PluginManager()

    @manager.register_plugin("plug_name")
    def test(_):
        pass

    @manager.register_plugin("plug_name2")
    def test2(_):
        pass

    assert {"plug_name", "plug_name2"} - set(manager.get_plugins("")) == set()
