import ArgosAnalysePlatform.plugins.analysis as analysis
from ArgosAnalysePlatform.database import DatabaseConnections

import pytest
from datetime import datetime


@pytest.fixture(name="mocked_app_boards_per_class")
def populate_data(memory_database):
    Boards = DatabaseConnections.tables["Boards"]
    database, app = memory_database

    for _, session in database.sessioniter("memory"):
        session.execute(
            Boards.insert(),
            [
                dict(
                    BoardID=1,
                    Class=0,
                    Time=datetime.fromisoformat("2021-01-01 01:00:00"),
                    ClassName="No Repair",
                ),
                dict(
                    BoardID=2,
                    Class=1,
                    Time=datetime.fromisoformat("2021-01-01 02:00:00"),
                    ClassName="Repair",
                ),
            ],
        )

        session.commit()

    return app


def test_get_boards_per_class(mocked_app_boards_per_class):

    result = mocked_app_boards_per_class.post(
        "/boards_by_class",
        json=dict(
            start_time="2021-01-01 00:00:00",
            end_time="2021-03-01 00:00:00",
            scanners=["memory"],
        ),
    ).json()

    assert result == {"0": [1, 50], "1": [1, 50]}


@pytest.fixture(name="mocked_app_available_types")
def populate_data_available_types(memory_database):

    Defects = DatabaseConnections.tables["Defects"]
    database, app = memory_database

    for _, session in database.sessioniter("memory"):
        session.execute(
            Defects.insert(),
            [
                dict(Type="A", SubType="B"),
                dict(Type="A", SubType="C"),
                dict(Type="B", SubType="A"),
            ],
        )

        session.commit()

    return app


def test_get_available_types(mocked_app_available_types):

    result = mocked_app_available_types.get(
        "/available_types",
        json=dict(
            scanners=["memory"],
        ),
    ).json()

    assert result == [
        {
            "children": [
                {"id": "B<**>C", "text": "All A"},
                {"id": "B", "text": "B"},
                {"id": "C", "text": "C"},
            ],
            "text": "A",
        },
        {
            "children": [
                {"id": "A", "text": "All B"},
                {"id": "A", "text": "A"},
            ],
            "text": "B",
        },
    ]


@pytest.fixture(name="mocked_app_lower_limits")
def populate_data_find_manual_simulate_rows(memory_database):
    Defects = DatabaseConnections.tables["Defects"]
    database, app = memory_database

    for _, session in database.sessioniter("memory"):
        session.execute(
            Defects.insert(),
            [
                dict(SubType="B", ArgosName="SB"),
                dict(SubType="C", ArgosName="SC"),
                dict(SubType="A", ArgosName="SA"),
            ],
        )

        session.commit()

    return app


def test_get_lower_limits(mocked_app_lower_limits):
    result = mocked_app_lower_limits.post(
        "/manual_simulate_rows",
        json=dict(
            scanners=["memory"],
            subtypes=["B"],
            old_limits={
                r"\vSB": {
                    "classlimit1_2": '"4"',
                    "classlimit2_3": '"5"',
                }
            },
        ),
    ).json()

    assert result == {"B": {"SB": {"1_2": 4, "2_3": 5}}}


@pytest.fixture(name="mocked_app_simulate_manual")
def populate_data_find_manual_simulate(memory_database):
    Defects = DatabaseConnections.tables["Defects"]
    database, app = memory_database

    for _, session in database.sessioniter("memory"):
        session.execute(
            Defects.insert(),
            [
                dict(SubType="B", ArgosName="SB"),
                dict(SubType="C", ArgosName="SC"),
                dict(SubType="A", ArgosName="SA"),
            ],
        )

        session.commit()

    return app


def test_simulate_manual(mocked_app_simulate_manual):
    result = mocked_app_simulate_manual.post(
        "/simulate_images/manual",
        json=dict(
            scanners=["memory"],
            subtypes=["B"],
            old_limits={
                r"\vSB": {
                    "classlimit1_2": '"4"',
                    "classlimit2_3": '"5"',
                }
            },
            limits={"SB": {"1_2": 1, "2_3": 2}},
        ),
    ).json()

    assert result == {"B": {"SB": {"1_2": 4, "2_3": 5}}}
